# set working directory (see https://stackoverflow.com/a/35842119)
dir = tryCatch({
  # script being sourced
  getSrcDirectory()[1]
}, error = function(e) {
  # script being run in RStudio
  dirname(rstudioapi::getActiveDocumentContext()$path)
})
setwd(dir)

library(fasttime)
library(data.table)
library(effsize)
library(sqldf)

# load global function
source("../global.R")

# read project information
tr_projects <- read.csv("data/tr_projects.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)
length(unique(tr_projects$gh_project_name))
# 1283

# parse timestamps
tr_projects$tr_first_build <- as.POSIXct(tr_projects$tr_first_build, tz="UTC")
tr_projects$tr_last_build <- as.POSIXct(tr_projects$tr_last_build, tz="UTC")
tr_projects$gh_first_commit <- as.POSIXct(tr_projects$gh_first_commit, tz="UTC")
tr_projects$gh_last_commit <- as.POSIXct(tr_projects$gh_last_commit, tz="UTC")

# filter tr_projects_sample
tr_projects_sample <- tr_projects[tr_projects$tr_timespan_before_ci >= 365 & tr_projects$tr_timespan_during_ci >= 365,]
length(unique(tr_projects_sample$gh_project_name))
# 544

mean(tr_projects_sample$tr_timespan_before_ci/365)
# 2.891756
sd(tr_projects_sample$tr_timespan_before_ci/365)
# 2.059573
median(tr_projects_sample$tr_timespan_before_ci/365)
# 2.235616

mean(tr_projects_sample$tr_timespan_during_ci/365)
# 3.170452
sd(tr_projects_sample$tr_timespan_during_ci/365)
# 1.129717
median(tr_projects_sample$tr_timespan_during_ci/365)
# 3.19863

# read project list after filtering using BigQuery (see SQL script "4_before_during_ci.sql")
tr_projects_sample <- read.csv("data/tr_projects_sample_filtered_2.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)
length(unique(tr_projects_sample$gh_project_name))
# 113


###########################
# read commits and merges #
###########################

###########
# commits #
###########
# read data for commits to default branch in sample projects before CI
tr_projects_sample_commits_default_branch_before_ci <- fread("data/tr_sample_commits_default_branch_before_ci.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
tr_projects_sample_commits_default_branch_before_ci$commit_date <- fastPOSIXct(tr_projects_sample_commits_default_branch_before_ci$commit_date, tz="UTC")
length(unique(tr_projects_sample_commits_default_branch_before_ci$project))
# 113

# read data for merges to default branch in sample projects before CI
tr_projects_sample_merges_default_branch_before_ci <- read.csv("data/tr_sample_merges_default_branch_before_ci.csv", header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE, strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
tr_projects_sample_merges_default_branch_before_ci$commit_date <- fastPOSIXct(tr_projects_sample_merges_default_branch_before_ci$commit_date, tz="UTC")
length(unique(tr_projects_sample_merges_default_branch_before_ci$project))
# 104

length(unique(c(tr_projects_sample_commits_default_branch_before_ci$project, tr_projects_sample_merges_default_branch_before_ci$project)))
# 113

projects_without_merges_before_ci <- setdiff(gsub("/", "_", unique(tr_projects_sample$gh_project_name)), unique(tr_projects_sample_merges_default_branch_before_ci$project))
length(projects_without_merges_before_ci)
# 9

##########
# merges #
##########
# read data for commits to default branch in sample projects during CI
tr_projects_sample_commits_default_branch_during_ci <- fread("data/tr_sample_commits_default_branch_during_ci.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
tr_projects_sample_commits_default_branch_during_ci$commit_date <- fastPOSIXct(tr_projects_sample_commits_default_branch_during_ci$commit_date, tz="UTC")
length(unique(tr_projects_sample_commits_default_branch_during_ci$project))
# 113

# read data for merges to default branch in sample projects during CI
tr_projects_sample_merges_default_branch_during_ci <- read.csv("data/tr_sample_merges_default_branch_during_ci.csv", header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE, strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
tr_projects_sample_merges_default_branch_during_ci$commit_date <- fastPOSIXct(tr_projects_sample_merges_default_branch_during_ci$commit_date, tz="UTC")
length(unique(tr_projects_sample_merges_default_branch_during_ci$project))
# 109

length(unique(c(tr_projects_sample_commits_default_branch_during_ci$project, tr_projects_sample_merges_default_branch_during_ci$project)))
# 113

tr_projects_without_merges_during_ci <- setdiff(gsub("/", "_", unique(tr_projects_sample$gh_project_name)), unique(tr_projects_sample_merges_default_branch_during_ci$project))
length(tr_projects_without_merges_during_ci)
# 4


###################
### COMMIT RATE ###
###################

###########################
# Global (Default Branch) #
###########################

###########
# commits #
###########
all_commits_per_week_default_branch_before <- get_all_commits_per_week(tr_projects_sample_commits_default_branch_before_ci, FALSE)
all_commits_per_week_default_branch_during <- get_all_commits_per_week(tr_projects_sample_commits_default_branch_during_ci, FALSE)

length(unique(tr_projects_sample_commits_default_branch_before_ci$project))
# 113
length(unique(tr_projects_sample_commits_default_branch_during_ci$project))
# 113
length(unique(c(tr_projects_sample_commits_default_branch_before_ci$project,tr_projects_sample_commits_default_branch_during_ci$project)))
# 113

commits_projects_before <- unique(all_commits_per_week_default_branch_before$project)
commits_projects_during <- unique(all_commits_per_week_default_branch_during$project)
commits_projects_before_during <- intersect(commits_projects_before, commits_projects_during)
length(commits_projects_before_during)
# 111

# projects that had only incomplete weeks of activity in either of the two timespans (see w_1 and w_n in paper)
commits_ignored_projects <- setdiff(gsub("/", "_", tr_projects_sample$gh_project_name), commits_projects_before_during)
length(commits_ignored_projects)
# 2

n <- nrow(all_commits_per_week_default_branch_before)
n
# 4960
n_0 <- nrow(all_commits_per_week_default_branch_before[all_commits_per_week_default_branch_before$commit_count == 0,])
n_0
# 2664
n_0/n*100
# 53.70968

n <- nrow(all_commits_per_week_default_branch_during)
n
# 5134
n_0 <- nrow(all_commits_per_week_default_branch_during[all_commits_per_week_default_branch_during$commit_count == 0,])
n_0
# 2622
n_0/n*100
# 51.07129


##########
# merges #
##########
all_merges_per_week_default_branch_before <- get_all_commits_per_week(tr_projects_sample_merges_default_branch_before_ci, FALSE)
all_merges_per_week_default_branch_during <- get_all_commits_per_week(tr_projects_sample_merges_default_branch_during_ci, FALSE)

length(unique(tr_projects_sample_merges_default_branch_before_ci$project))
# 104
length(unique(tr_projects_sample_merges_default_branch_during_ci$project))
# 109
length(unique(c(tr_projects_sample_merges_default_branch_before_ci$project,tr_projects_sample_merges_default_branch_during_ci$project)))
# 112

merges_projects_before <- unique(all_merges_per_week_default_branch_before$project)
merges_projects_during <- unique(all_merges_per_week_default_branch_during$project)
merges_projects_before_during <- intersect(merges_projects_before, merges_projects_during)
length(merges_projects_before_during)
# 93

# projects that had only incomplete weeks of activity in either of the two timespans (see w_1 and w_n in paper)
merges_ignored_projects <- setdiff(gsub("/", "_", tr_projects_sample$gh_project_name), merges_projects_before_during)
length(merges_ignored_projects)
# 20

n <- nrow(all_merges_per_week_default_branch_before)
n
# 3495 
n_0 <- nrow(all_merges_per_week_default_branch_before[all_merges_per_week_default_branch_before$commit_count == 0,])
n_0
# 2559
n_0/n*100
# 73.21888

n <- nrow(all_merges_per_week_default_branch_during)
n
# 4183
n_0 <- nrow(all_merges_per_week_default_branch_during[all_merges_per_week_default_branch_during$commit_count == 0,])
n_0
# 2891
n_0/n*100
# 69.11308 


##################################
# exclude weeks without activity #
##################################
all_commits_per_week_default_branch_before <- all_commits_per_week_default_branch_before[all_commits_per_week_default_branch_before$commit_count > 0,]
all_commits_per_week_default_branch_during <- all_commits_per_week_default_branch_during[all_commits_per_week_default_branch_during$commit_count > 0,]
all_merges_per_week_default_branch_before <- all_merges_per_week_default_branch_before[all_merges_per_week_default_branch_before$commit_count > 0,]
all_merges_per_week_default_branch_during <- all_merges_per_week_default_branch_during[all_merges_per_week_default_branch_during$commit_count > 0,]


######################################
# exclude projects with too few data #
######################################
length(unique(c(
  tr_projects_sample_commits_default_branch_before_ci$project,
  tr_projects_sample_commits_default_branch_during_ci$project,
  tr_projects_sample_merges_default_branch_before_ci$project,
  tr_projects_sample_merges_default_branch_during_ci$project
)))
# 113

projects_before_during <- intersect(commits_projects_before_during, merges_projects_before_during)
length(projects_before_during)
# 93

# projects that had only incomplete weeks of activity in either of the two timespans (see w_1 and w_n in paper)
ignored_projects <- unique(c(commits_ignored_projects, merges_ignored_projects))
length(ignored_projects)
# 20

# remove projects without data for both timespans
all_commits_per_week_default_branch_before <- all_commits_per_week_default_branch_before[all_commits_per_week_default_branch_before$project %in% projects_before_during,]
all_commits_per_week_default_branch_during <- all_commits_per_week_default_branch_during[all_commits_per_week_default_branch_during$project %in% projects_before_during,]
all_merges_per_week_default_branch_before <- all_merges_per_week_default_branch_before[all_merges_per_week_default_branch_before$project %in% projects_before_during,]
all_merges_per_week_default_branch_during <- all_merges_per_week_default_branch_during[all_merges_per_week_default_branch_during$project %in% projects_before_during,]

tr_projects_sample_commits_default_branch_before_ci <- tr_projects_sample_commits_default_branch_before_ci[tr_projects_sample_commits_default_branch_before_ci$project %in% projects_before_during,]
tr_projects_sample_commits_default_branch_during_ci <- tr_projects_sample_commits_default_branch_during_ci[tr_projects_sample_commits_default_branch_during_ci$project %in% projects_before_during,]

tr_projects_sample_merges_default_branch_before_ci <- tr_projects_sample_merges_default_branch_before_ci[tr_projects_sample_merges_default_branch_before_ci$project %in% projects_before_during,]
tr_projects_sample_merges_default_branch_during_ci <- tr_projects_sample_merges_default_branch_during_ci[tr_projects_sample_merges_default_branch_during_ci$project %in% projects_before_during,]

# get contributors per project
project_contributors <- unique.data.frame(tr_projects_sample_commits_default_branch_before_ci[,c("project", "author_email")])
names(project_contributors) <- c("project", "email")
tmp <- unique.data.frame(tr_projects_sample_commits_default_branch_before_ci[,c("project", "commit_email")])
names(tmp) <- c("project", "email")
project_contributors <- unique.data.frame(rbind(project_contributors, tmp))
tmp <- unique.data.frame(tr_projects_sample_merges_default_branch_before_ci[,c("project", "author_email")])
names(tmp) <- c("project", "email")
project_contributors <- unique.data.frame(rbind(project_contributors, tmp))
tmp <- unique.data.frame(tr_projects_sample_merges_default_branch_before_ci[,c("project", "commit_email")])
names(tmp) <- c("project", "email")
project_contributors <- unique.data.frame(rbind(project_contributors, tmp))

project_contributors <- sqldf("SELECT project, COUNT(DISTINCT email) as contributors FROM project_contributors GROUP BY project ORDER BY contributors DESC")
summary(project_contributors$contributors)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.00    9.00   14.00   21.83   22.00  220.00

IQR(project_contributors$contributors)
# 13

small_projects <- project_contributors[project_contributors$contributors<=14,]$project
length(small_projects)
# 47

large_projects <- project_contributors[project_contributors$contributors>14,]$project
length(large_projects)
# 46


######################################
# analysis #
######################################

#############################
# Projects (Default Branch) #
#############################

###########
# commits #
###########
median_commit_rate_default_branch_before <- get_median_commit_rate(tr_projects_sample_commits_default_branch_before_ci, TRUE)
median_commit_rate_default_branch_during <- get_median_commit_rate(tr_projects_sample_commits_default_branch_during_ci, TRUE)

median_commit_rate_default_branch_before_during <- merge(median_commit_rate_default_branch_before, median_commit_rate_default_branch_during, by="project")
names(median_commit_rate_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(median_commit_rate_default_branch_before_during$before) | is.na(median_commit_rate_default_branch_before_during$during))
median_commit_rate_default_branch_before_during <- median_commit_rate_default_branch_before_during[remove_empty_rows,]

nrow(median_commit_rate_default_branch_before_during)
# 93
summary(median_commit_rate_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   2.000   2.500   3.473   4.500  26.000
IQR(median_commit_rate_default_branch_before_during$before)
# 2.5
summary(median_commit_rate_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   2.000   2.000   3.274   4.000  28.000 
IQR(median_commit_rate_default_branch_before_during$during)
# 2

pdf("figures/commit_rate_project_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="commit rate (default branch)",
        names=c("before_ci", "during_ci"),
        median_commit_rate_default_branch_before_during$before, median_commit_rate_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(median_commit_rate_default_branch_before_during$during,
            median_commit_rate_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) 
# V = 1342, p-value = 0.66
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(median_commit_rate_default_branch_before_during$during,
            median_commit_rate_default_branch_before_during$before,
            paired=TRUE)
# delta estimate: -0.01526188 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.1776505  0.1479359

cohen.d(median_commit_rate_default_branch_before_during$during,
        median_commit_rate_default_branch_before_during$before,
        paired=TRUE)
# d estimate: -0.09281826 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.3823000  0.1966635


##########
# merges #
##########
median_merge_rate_default_branch_before <- get_median_commit_rate(tr_projects_sample_merges_default_branch_before_ci, TRUE)
median_merge_rate_default_branch_during <- get_median_commit_rate(tr_projects_sample_merges_default_branch_during_ci, TRUE)

median_merge_rate_default_branch_before_during <- merge(median_merge_rate_default_branch_before, median_merge_rate_default_branch_during, by="project")
names(median_merge_rate_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(median_merge_rate_default_branch_before_during$before) | is.na(median_merge_rate_default_branch_before_during$during))
median_merge_rate_default_branch_before_during <- median_merge_rate_default_branch_before_during[remove_empty_rows,]

nrow(median_merge_rate_default_branch_before_during)
# 93
summary(median_merge_rate_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.403   2.000   5.000 
IQR(median_merge_rate_default_branch_before_during$before)
# 1
summary(median_merge_rate_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.522   2.000   7.500 
IQR(median_merge_rate_default_branch_before_during$during)
# 1

pdf("figures/merge_rate_project_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="merge rate (default branch)",
        names=c("before_ci", "during_ci"),
        median_merge_rate_default_branch_before_during$before, median_merge_rate_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(median_merge_rate_default_branch_before_during$during,
            median_merge_rate_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) 
# V = 803.5, p-value = 0.1017
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(median_merge_rate_default_branch_before_during$during,
        median_merge_rate_default_branch_before_during$before,
        paired=TRUE)
# delta estimate: 0.08706209 (negligible)
# 95 percent confidence interval:
#   inf         sup 
# -0.06195484  0.23228214 

cohen.d(median_merge_rate_default_branch_before_during$during,
        median_merge_rate_default_branch_before_during$before,
        paired=TRUE)
# d estimate: 0.1838115 (negligible)
# d estimate: 0.1746437 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.1152333  0.4645208 


#############################
### COMMIT SIZE (Breadth) ###
#############################

#############################
# Projects (Default Branch) #
#############################

###########
# commits #
###########
median_commit_file_count_default_branch_before <- get_median_file_count(tr_projects_sample_commits_default_branch_before_ci)
median_commit_file_count_default_branch_during <- get_median_file_count(tr_projects_sample_commits_default_branch_during_ci)

median_commit_file_count_default_branch_before_during <- merge(median_commit_file_count_default_branch_before, median_commit_file_count_default_branch_during, by="project")
names(median_commit_file_count_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(median_commit_file_count_default_branch_before_during$before) | is.na(median_commit_file_count_default_branch_before_during$during))
median_commit_file_count_default_branch_before_during <- median_commit_file_count_default_branch_before_during[remove_empty_rows,]

nrow(median_commit_file_count_default_branch_before_during)
# 93
summary(median_commit_file_count_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.796   2.000  39.000
IQR(median_commit_file_count_default_branch_before_during$before)
# 1

summary(median_commit_file_count_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.565   2.000  27.000
IQR(median_commit_file_count_default_branch_before_during$during)
# 1

pdf("figures/commit_breadth_projects_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="commit breadth (default branch)",
        names=c("before_ci", "during_ci"),
        median_commit_file_count_default_branch_before_during$before, median_commit_file_count_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(median_commit_file_count_default_branch_before_during$during,
            median_commit_file_count_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T, exact=F) 
# V = 204, p-value = 0.05325
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(median_commit_file_count_default_branch_before_during$during,
        median_commit_file_count_default_branch_before_during$before,
        paired=TRUE
)
# delta estimate: -0.07399699 (negligible)
# 95 percent confidence interval:
#   inf         sup 
# -0.20820483  0.06294577 

cohen.d(median_commit_file_count_default_branch_before_during$during,
        median_commit_file_count_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: -0.16894 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.4587817  0.120901


##########
# merges #
##########
median_merge_file_count_default_branch_before <- get_median_file_count(tr_projects_sample_merges_default_branch_before_ci)
median_merge_file_count_default_branch_during <- get_median_file_count(tr_projects_sample_merges_default_branch_during_ci)

median_merge_file_count_default_branch_before_during <- merge(median_merge_file_count_default_branch_before, median_merge_file_count_default_branch_during, by="project")
names(median_merge_file_count_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(median_merge_file_count_default_branch_before_during$before) | is.na(median_merge_file_count_default_branch_before_during$during))
median_merge_file_count_default_branch_before_during <- median_merge_file_count_default_branch_before_during[remove_empty_rows,]

nrow(median_merge_file_count_default_branch_before_during)
# 93
summary(median_merge_file_count_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   2.000   2.000   2.839   3.000  61.000 
IQR(median_merge_file_count_default_branch_before_during$before)
# 1

summary(median_merge_file_count_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   2.000   2.000   1.995   2.000   4.000 
IQR(median_merge_file_count_default_branch_before_during$during)
# 0

pdf("figures/merge_breadth_projects_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="merge breadth (default branch)",
        names=c("before_ci", "during_ci"),
        median_merge_file_count_default_branch_before_during$before, median_merge_file_count_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(median_merge_file_count_default_branch_before_during$during,
            median_merge_file_count_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) 
# V = 464.5, p-value = 0.03856
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(median_merge_file_count_default_branch_before_during$during,
        median_merge_file_count_default_branch_before_during$before,
        paired=TRUE
)
# delta estimate: -0.09527113 (negligible)
# 95 percent confidence interval:
#   inf         sup 
# -0.24364290  0.05745801 

cohen.d(median_merge_file_count_default_branch_before_during$during,
        median_merge_file_count_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: -0.1390636 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.4287391  0.1506119


###########################
### COMMIT SIZE (Depth) ###
###########################

# add line count to commits and merges before and during CI
tr_projects_sample_commits_default_branch_before_ci$line_count <- tr_projects_sample_commits_default_branch_before_ci$lines_added + tr_projects_sample_commits_default_branch_before_ci$lines_deleted
tr_projects_sample_merges_default_branch_before_ci$line_count <- tr_projects_sample_merges_default_branch_before_ci$lines_added + tr_projects_sample_merges_default_branch_before_ci$lines_deleted
tr_projects_sample_commits_default_branch_during_ci$line_count <- tr_projects_sample_commits_default_branch_during_ci$lines_added + tr_projects_sample_commits_default_branch_during_ci$lines_deleted
tr_projects_sample_merges_default_branch_during_ci$line_count <- tr_projects_sample_merges_default_branch_during_ci$lines_added + tr_projects_sample_merges_default_branch_during_ci$lines_deleted

# add normalized line count
tr_projects_sample_commits_default_branch_before_ci$normalized_line_count <- tr_projects_sample_commits_default_branch_before_ci$line_count/tr_projects_sample_commits_default_branch_before_ci$file_count
tr_projects_sample_commits_default_branch_during_ci$normalized_line_count <- tr_projects_sample_commits_default_branch_during_ci$line_count/tr_projects_sample_commits_default_branch_during_ci$file_count
tr_projects_sample_merges_default_branch_before_ci$normalized_line_count <- tr_projects_sample_merges_default_branch_before_ci$line_count/tr_projects_sample_merges_default_branch_before_ci$file_count
tr_projects_sample_merges_default_branch_during_ci$normalized_line_count <- tr_projects_sample_merges_default_branch_during_ci$line_count/tr_projects_sample_merges_default_branch_during_ci$file_count


#############################
# Projects (Default Branch) #
#############################

###########
# commits #
###########
commits_median_normalized_line_count_default_branch_before <- get_median_normalized_line_count_all(tr_projects_sample_commits_default_branch_before_ci)
commits_median_normalized_line_count_default_branch_during<- get_median_normalized_line_count_all(tr_projects_sample_commits_default_branch_during_ci)

commits_median_normalized_line_count_default_branch_before_during <- merge(commits_median_normalized_line_count_default_branch_before, commits_median_normalized_line_count_default_branch_during, by="project")
names(commits_median_normalized_line_count_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(commits_median_normalized_line_count_default_branch_before_during$before) | is.na(commits_median_normalized_line_count_default_branch_before_during$during))
commits_median_normalized_line_count_default_branch_before_during <- commits_median_normalized_line_count_default_branch_before_during[remove_empty_rows,]

nrow(commits_median_normalized_line_count_default_branch_before_during)
# 93
summary(commits_median_normalized_line_count_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 2.000   6.000   7.750   9.916  10.750  58.361
IQR(commits_median_normalized_line_count_default_branch_before_during$before)
# 4.75

summary(commits_median_normalized_line_count_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 2.000   5.667   7.000   8.625  10.000  55.702 
IQR(commits_median_normalized_line_count_default_branch_before_during$during)
# 4.333333

pdf("figures/commit_depth_projects_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="commit depth (default branch)",
        names=c("before_ci", "during_ci"),
        commits_median_normalized_line_count_default_branch_before_during$before, commits_median_normalized_line_count_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(commits_median_normalized_line_count_default_branch_before_during$during,
            commits_median_normalized_line_count_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) 
# V = 1091, p-value = 0.0007926
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(commits_median_normalized_line_count_default_branch_before_during$during,
            commits_median_normalized_line_count_default_branch_before_during$before,
            paired=TRUE
)
# delta estimate: -0.1025552 (negligible)
# 95 percent confidence interval:
#   inf         sup 
# -0.26253828  0.06291536 

cohen.d(commits_median_normalized_line_count_default_branch_before_during$during,
        commits_median_normalized_line_count_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: -0.3573397 (small)
# 95 percent confidence interval:
#   inf         sup 
# -0.64896565 -0.06571379  


##########
# merges #
##########
merges_median_normalized_line_count_default_branch_before <- get_median_normalized_line_count_all(tr_projects_sample_merges_default_branch_before_ci)
merges_median_normalized_line_count_default_branch_during<- get_median_normalized_line_count_all(tr_projects_sample_merges_default_branch_during_ci)

merges_median_normalized_line_count_default_branch_before_during <- merge(merges_median_normalized_line_count_default_branch_before, merges_median_normalized_line_count_default_branch_during, by="project")
names(merges_median_normalized_line_count_default_branch_before_during) <- c("project", "before", "sd_before", "during", "sd_during")
remove_empty_rows <- !(is.na(merges_median_normalized_line_count_default_branch_before_during$before) | is.na(merges_median_normalized_line_count_default_branch_before_during$during))
merges_median_normalized_line_count_default_branch_before_during <- merges_median_normalized_line_count_default_branch_before_during[remove_empty_rows,]

nrow(merges_median_normalized_line_count_default_branch_before_during)
# 93
summary(merges_median_normalized_line_count_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.00    7.31   10.00   16.76   16.71  128.3
IQR(merges_median_normalized_line_count_default_branch_before_during$before)
# 9.404762
summary(merges_median_normalized_line_count_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 2.000   6.958   9.500  12.428  14.500  95.250 
IQR(merges_median_normalized_line_count_default_branch_before_during$during)
# 7.541667

pdf("figures/merge_depth_projects_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="merge depth (default branch)",
        names=c("before_ci", "during_ci"),
        merges_median_normalized_line_count_default_branch_before_during$before, merges_median_normalized_line_count_default_branch_before_during$during, outline=FALSE)
dev.off()

wilcox.test(merges_median_normalized_line_count_default_branch_before_during$during,
            merges_median_normalized_line_count_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) 
# V = 1636.5, p-value = 0.09859
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(merges_median_normalized_line_count_default_branch_before_during$during,
            merges_median_normalized_line_count_default_branch_before_during$before,
            paired=TRUE
)
# delta estimate: -0.07607816 (negligible)
# 95 percent confidence interval:
#   inf         sup 
# -0.23672472  0.08861756 

cohen.d(merges_median_normalized_line_count_default_branch_before_during$during,
        merges_median_normalized_line_count_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: -0.214717 (small)
# 95 percent confidence interval:
#   inf         sup 
# -0.50487552  0.07544153 


###################
### MERGE RATIO ###
###################

#############################
# Projects (default branch) #
#############################
project_merge_ratio_default_branch_before <- get_project_merge_ratio(tr_projects_sample_commits_default_branch_before_ci, tr_projects_sample_merges_default_branch_before_ci)
project_merge_ratio_default_branch_during <- get_project_merge_ratio(tr_projects_sample_commits_default_branch_during_ci, tr_projects_sample_merges_default_branch_during_ci)

project_merge_ratio_default_branch_before_during <- merge(project_merge_ratio_default_branch_before, project_merge_ratio_default_branch_during, by="project")
names(project_merge_ratio_default_branch_before_during) <- c("project", "before", "during")

summary(project_merge_ratio_default_branch_before_during$before)
# Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# 0.009804 0.084362 0.141414 0.160371 0.210526 0.687500
IQR(project_merge_ratio_default_branch_before_during$before[!is.na(project_merge_ratio_default_branch_before_during$before)])
# 0.1261642

summary(project_merge_ratio_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.01149 0.12451 0.19251 0.19839 0.26087 0.40741
IQR(project_merge_ratio_default_branch_before_during$during[!is.na(project_merge_ratio_default_branch_before_during$during)])
# 0.1363636

pdf("figures/merge_ratio_projects_default_branch.pdf", bg="transparent", width=8, height=5)
boxplot(main="merge ratio (default branch)",
        names=c("before_ci", "during_ci"),
        project_merge_ratio_default_branch_before_during$before, project_merge_ratio_default_branch_before_during$during)
dev.off()

wilcox.test(project_merge_ratio_default_branch_before_during$during,
            project_merge_ratio_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# V = 3308, p-value = 5.364e-06
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(project_merge_ratio_default_branch_before_during$during,
        project_merge_ratio_default_branch_before_during$before,
        paired=TRUE
)
# delta estimate: 0.281073 (small)
# 95 percent confidence interval:
#   inf       sup 
# 0.1200952 0.4276488 

cohen.d(project_merge_ratio_default_branch_before_during$during,
        project_merge_ratio_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: 0.3579206 (small)
# 95 percent confidence interval:
#   inf        sup 
# 0.06628717 0.64955394

############################
# small vs. large projects #
############################

summary(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$before)
# Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# 0.009804 0.060950 0.121951 0.148384 0.192105 0.687500
IQR(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$before)
# 0.1311556

summary(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.01657 0.10856 0.19048 0.18196 0.25000 0.32353 
IQR(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$during)
# 0.1414409

wilcox.test(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$during,
            project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$before,
            alternative="two.sided", paired=T, correct=T, exact=F)
# V = 831, p-value = 0.001533
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$during,
            project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% small_projects,]$before,
            paired=TRUE
)
# delta estimate: 0.2838388 (small)
# 95 percent confidence interval:
#   inf        sup 
# 0.05955538 0.48084190 

summary(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.02796 0.10392 0.14700 0.17262 0.21750 0.45238
IQR(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$before)
# 0.1135863

summary(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.01149 0.14834 0.19803 0.21517 0.28771 0.40741
IQR(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$during)
# 0.139371

wilcox.test(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$during,
            project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$before,
            alternative="two.sided", paired=T, correct=T, exact=F)
# V = 845, p-value = 0.0008959
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$during,
            project_merge_ratio_default_branch_before_during[project_merge_ratio_default_branch_before_during$project %in% large_projects,]$before,
            paired=TRUE
)
# delta estimate: 0.2873346 (small)
# 95 percent confidence interval:
#   inf        sup 
# 0.05534375 0.48989089 


#####################################################
# OBSERVATION: More merges during CI. Also more PR? #
#####################################################
project_pr_ratio_default_branch_before <- get_project_pr_ratio(tr_projects_sample_merges_default_branch_before_ci)
project_pr_ratio_default_branch_during <- get_project_pr_ratio(tr_projects_sample_merges_default_branch_during_ci)

project_pr_ratio_default_branch_before_during <- merge(project_pr_ratio_default_branch_before, project_pr_ratio_default_branch_during, by="project")
names(project_pr_ratio_default_branch_before_during) <- c("project", "before", "during")

summary(project_pr_ratio_default_branch_before_during$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0000  0.4615  0.7778  0.6783  0.9231  1.0000
IQR(project_pr_ratio_default_branch_before_during$before)
# 0.4615385

summary(project_pr_ratio_default_branch_before_during$during)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.03226 0.76471 0.88235 0.80307 0.97619 1.00000 
IQR(project_pr_ratio_default_branch_before_during$during)
# 0.2114846

wilcox.test(project_pr_ratio_default_branch_before_during$during,
            project_pr_ratio_default_branch_before_during$before,
            alternative="two.sided", paired=T, correct=T) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# V = 2676.5, p-value = 7.077e-05
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(project_pr_ratio_default_branch_before_during$during,
        project_pr_ratio_default_branch_before_during$before,
        paired=TRUE
)
# delta estimate: 0.2444213 (small)
# 95 percent confidence interval:
#   inf        sup 
# 0.07756414 0.39795917

cohen.d(project_pr_ratio_default_branch_before_during$during,
        project_pr_ratio_default_branch_before_during$before,
        paired=TRUE
)
# d estimate: 0.4404185 (small)
# 95 percent confidence interval:
#   inf       sup 
# 0.1476059 0.7332310


####### PLOTS ######

###########
# commits #
###########

pdf("figures/projects_default_branch_merge_ratio.pdf", bg="transparent", width=6, height=6)
par(
  bg="white",
  #mar = c(3, 3, 3, 1)+0.1, # subplot margins (bottom, left, top, right)
  #  omi = c(0.0, 0.0, 0.0, 0.0),  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.3,
  cex.main=1.3,
  cex.sub=1,
  cex.lab=1,
  cex.axis=1
)

boxplot(main="merge ratio",
        names=c("before_ci", "during_ci"),
        project_merge_ratio_default_branch_before_during$before,
        project_merge_ratio_default_branch_before_during$during,
        outline=TRUE,
        ylim=c(0,1),
        yaxt="n"
)
axis(2, at=seq(0.0, 1.0, by=0.1), labels=c("0.0", seq(0.1, 0.9, by=0.1), "1.0"), las=2)

par(mfrow=c(1, 1))
dev.off()


#library(vioplot)
devtools::install_github("TomKellyGenetics/vioplotx")
pdf("figures/projects_default_branch_merge_ratio_violin.pdf", bg="transparent", width=8, height=6)
par(
  bg="white",
  #mar = c(3, 3, 3, 1)+0.1, # subplot margins (bottom, left, top, right)
  #  omi = c(0.0, 0.0, 0.0, 0.0),  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.1,
  cex.main=1.3,
  cex.sub=1.1,
  cex.lab=1,1,
  cex.axis=1.1
)

vioplotx::vioplotx(
  project_merge_ratio_default_branch_before_during$before,
  project_merge_ratio_default_branch_before_during$during,
  col="gray90",
  pchMed=4,
  rectCol="white",
  colMed="black",
  horizontal=FALSE,
  cex.names=1.3,
  wex=1,
  names=c("before_ci", "during_ci"),
  ylim=c(0, 0.7),
  yaxt="n"
)
#abline(v=median(project_merge_ratio_default_branch_before_during$during), lty=3)
segments(1, median(project_merge_ratio_default_branch_before_during$before),
         0, median(project_merge_ratio_default_branch_before_during$before),
         lty=3
)
segments(2, median(project_merge_ratio_default_branch_before_during$during),
         0, median(project_merge_ratio_default_branch_before_during$during),
         lty=3
)
axis(2, at=seq(0, 0.7, by=0.1), labels=seq(0, 0.7, by=0.1), las=2)
title("Merge ratio (CI sample)")

par(mfrow=c(1, 1))
dev.off()


pdf("figures/projects_default_branch_pr_ratio.pdf", bg="transparent", width=6, height=6)
par(
  bg="white",
  #mar = c(3, 3, 3, 1)+0.1, # subplot margins (bottom, left, top, right)
  #  omi = c(0.0, 0.0, 0.0, 0.0),  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.3,
  cex.main=1.3,
  cex.sub=1,
  cex.lab=1,
  cex.axis=1
)

boxplot(main="pull request ratio",
        names=c("before_ci", "during_ci"),
        project_pr_ratio_default_branch_before_during$before,
        project_pr_ratio_default_branch_before_during$during,
        outline=TRUE,
        ylim=c(0,1),
        yaxt="n"
)
axis(2, at=seq(0.0, 1.0, by=0.1), labels=c("0.0", seq(0.1, 0.9, by=0.1), "1.0"), las=2)

par(mfrow=c(1, 1))
dev.off()


#library(vioplot)
devtools::install_github("TomKellyGenetics/vioplotx")
pdf("figures/projects_default_branch_pr_ratio_violin.pdf", bg="transparent", width=8, height=6)
par(
  bg="white",
  #mar = c(3, 3, 3, 1)+0.1, # subplot margins (bottom, left, top, right)
  #  omi = c(0.0, 0.0, 0.0, 0.0),  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.1,
  cex.main=1.3,
  cex.sub=1.1,
  cex.lab=1,1,
  cex.axis=1.1
)

vioplotx::vioplotx(
  project_pr_ratio_default_branch_before_during$before,
  project_pr_ratio_default_branch_before_during$during,
  col="gray90",
  pchMed=4,
  rectCol="white",
  colMed="black",
  horizontal=FALSE,
  cex.names=1.3,
  wex=1,
  names=c("before_ci", "during_ci"),
  ylim=c(0, 1.0),
  yaxt="n"
)
#abline(v=median(project_pr_ratio_default_branch_before_during$during), lty=3)
segments(1, median(project_pr_ratio_default_branch_before_during$before),
         0, median(project_pr_ratio_default_branch_before_during$before),
         lty=3
)
segments(2, median(project_pr_ratio_default_branch_before_during$during),
         0, median(project_pr_ratio_default_branch_before_during$during),
         lty=3
)
axis(2, at=seq(0, 1.0, by=0.1), labels=seq(0, 1.0, by=0.1), las=2)
title("Pull request ratio (CI sample)")

par(mfrow=c(1, 1))
dev.off()



#################################
# Analyze the comparison sample #
#################################

projects_comparison <- fread("data/comparison_project_sample_800.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
nrow(projects_comparison)
# 800

branches_comparison <- fread("data/comparison_sample_branches.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
length(unique(branches_comparison$project))
# 793 -> retrieval failed for 7 projects (e.g., projects are private now)

###########
# commits #
###########
# read data for commits to default branch in sample projects before CI
commits_default_branch_before_mid <- fread("data/commits_default_branch_before_mid.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
commits_default_branch_before_mid$commit_date <- fastPOSIXct(commits_default_branch_before_mid$commit_date, tz="UTC")
length(unique(commits_default_branch_before_mid$project))
# 312

# read data for merges to default branch in sample projects before CI
merges_default_branch_before_mid <- read.csv("data/merges_default_branch_before_mid.csv", header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE, strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
merges_default_branch_before_mid$commit_date <- fastPOSIXct(merges_default_branch_before_mid$commit_date, tz="UTC")
length(unique(merges_default_branch_before_mid$project))
# 178

length(unique(c(commits_default_branch_before_mid$project, merges_default_branch_before_mid$project)))
# 312

projects_without_commits_before_mid <- setdiff(gsub("/", "_", unique(projects_comparison$repo_name)), unique(commits_default_branch_before_mid$project))
length(projects_without_commits_before_mid)
# 488

projects_without_merges_before_mid <- setdiff(gsub("/", "_", unique(projects_comparison$repo_name)), unique(merges_default_branch_before_mid$project))
length(projects_without_merges_before_mid)
# 622


##########
# merges #
##########
# read data for commits to default branch in sample projects during CI
commits_default_branch_after_mid <- fread("data/commits_default_branch_after_mid.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
commits_default_branch_after_mid$commit_date <- fastPOSIXct(commits_default_branch_after_mid$commit_date, tz="UTC")
length(unique(commits_default_branch_after_mid$project))
# 290

# read data for merges to default branch in sample projects during CI
merges_default_branch_after_mid <- read.csv("data/merges_default_branch_after_mid.csv", header=TRUE, sep=",", quote="\"", stringsAsFactors=FALSE, strip.white=TRUE, encoding="UTF-8", na.strings=c("", "null"))
# parse timestamps
merges_default_branch_after_mid$commit_date <- fastPOSIXct(merges_default_branch_after_mid$commit_date, tz="UTC")
length(unique(merges_default_branch_after_mid$project))
# 165

length(unique(c(commits_default_branch_after_mid$project, merges_default_branch_after_mid$project)))
# 291

projects_without_commits_after_mid <- setdiff(gsub("/", "_", unique(projects_comparison$repo_name)), unique(commits_default_branch_after_mid$project))
length(projects_without_commits_before_mid)
# 488

projects_without_merges_after_mid <- setdiff(gsub("/", "_", unique(projects_comparison$repo_name)), unique(merges_default_branch_after_mid$project))
length(projects_without_merges_before_mid)
# 622

# remove projects without data for both timespans
  
projects_before <- unique(intersect(commits_default_branch_before_mid$project, merges_default_branch_before_mid$project))
length(projects_before)  
# 178

projects_after <- unique(intersect(commits_default_branch_after_mid$project, merges_default_branch_after_mid$project))
length(projects_before) 
# 178

projects_before_after <- intersect(projects_before, projects_after)
length(projects_before_after)
# 130

# manually check for CI config files in projects
paste0("https://github.com/", gsub("_", "/", projects_before_after))
#  [1] "https://github.com/socialsensor/graphdb-benchmarks"                
#  [2] "https://github.com/SharedKnowledge/SharkFW"                        
#  [3] "https://github.com/bigapple520520/bigapple"                        
#  [4] "https://github.com/googlemaps/google-maps-services-java" -> Travis        
#  [5] "https://github.com/swagger-api/swagger-parser" -> Travis                     
#  [6] "https://github.com/ai212983/android-spinnerwheel"                
#  [7] "https://github.com/MassiveCraft/Factions"                          
#  [8] "https://github.com/ngageoint/geowave" -> Travis                            
#  [9] "https://github.com/mihnita/ansi-econsole"                          
# [10] "https://github.com/JetBrains/intellij-haxe" -> Travis                   
# [11] "https://github.com/janinko/ghprb"                                  
# [12] "https://github.com/jcabi/jcabi-mysql-maven-plugin" -> Travis            
# [13] "https://github.com/Quetzi/Morpheus"                                
# [14] "https://github.com/iconara/rubydoop" -> Travis                             
# [15] "https://github.com/libetl/soundtransform" -> Travis                         
# [16] "https://github.com/MovingBlocks/TerasologyLauncher"                
# [17] "https://github.com/naver/ngrinder" -> Travis                               
# [18] "https://github.com/vkostyukov/la4j" -> Travis                           
# [19] "https://github.com/greenrobot/EventBus" -> Travis                          
# [20] "https://github.com/pcingola/SnpEff"                                
# [21] "https://github.com/graphaware/neo4j-algorithms" -> Travis               
# [22] "https://github.com/chrisjenx/Calligraphy"                          
# [23] "https://github.com/iPaulPro/aFileChooser"                          
# [24] "https://github.com/mirkosertic/GameComposer" -> Travis                      
# [25] "https://github.com/yegor256/s3auth" -> Travis                           
# [26] "https://github.com/wikimedia/apps-android-wikipedia"               
# [27] "https://github.com/QuantumBadger/RedReader" -> Travis                       
# [28] "https://github.com/JakeWharton/hugo" -> Travis                             
# [29] "https://github.com/cyclestreets/android"-> Travis                           
# [30] "https://github.com/square/protoparser"-> Travis                             
# [31] "https://github.com/JakeWharton/madge" -> Travis                              
# [32] "https://github.com/eclipse/smarthome" -> Travis                                
# [33] "https://github.com/Todd-Davies/ProgressWheel"                      
# [34] "https://github.com/couchbase/couchbase-lite-android"               
# [35] "https://github.com/gluster/glusterfs-hadoop"                       
# [36] "https://github.com/CompendiumNG/CompendiumNG" -> Travis                      
# [37] "https://github.com/perlundq/yajsync"                               
# [38] "https://github.com/square/mortar" -> Travis                                   
# [39] "https://github.com/romannurik/muzei"                               
# [40] "https://github.com/michaelklishin/quartz-mongodb" -> Travis                   
# [41] "https://github.com/mcharmas/Android-ReactiveLocation"              
# [42] "https://github.com/onepf/openiab"                                  
# [43] "https://github.com/linkedin/parseq" -> Travis                                
# [44] "https://github.com/realityforge/gwt-appcache" -> Travis                      
# [45] "https://github.com/cloudfoundry/java-test-applications"            
# [46] "https://github.com/ukanth/afwall" -> Travis                                  
# [47] "https://github.com/hdodenhof/CircleImageView"                      
# [48] "https://github.com/jbosstm/narayana"                               
# [49] "https://github.com/versioneye/versioneye/maven/plugin" (should be versioneye/versioneye_maven_plugin)           
# [50] "https://github.com/Netflix/exhibitor" -> Travis                              
# [51] "https://github.com/WASdev/ci.maven" -> Travis                                 
# [52] "https://github.com/SeaCloudsEU/SeaCloudsPlatform" -> Travis                 
# [53] "https://github.com/arx-deidentifier/arx"                           
# [54] "https://github.com/pac4j/vertx-pac4j" -> Travis                             
# [55] "https://github.com/OpenLMIS/open-lmis"                             
# [56] "https://github.com/btrplace/scheduler" -> Travis                            
# [57] "https://github.com/greenlaw110/Rythm"                              
# [58] "https://github.com/debop/hibernate-redis" -> Travis                          
# [59] "https://github.com/tehapo/Clara" -> Travis                                   
# [60] "https://github.com/Mahoney/slf4j-test" -> Travis                              
# [61] "https://github.com/pagehelper/Mybatis-PageHelper" -> Travis                  
# [62] "https://github.com/buddycloud/buddycloud-media-server" -> Travis             
# [63] "https://github.com/openshift/jenkins-cloud-plugin"                 
# [64] "https://github.com/greese/dasein-cloud-core"                       
# [65] "https://github.com/Firebase/geofire-java" -> Travis                           
# [66] "https://github.com/felHR85/UsbSerial" -> Travis                               
# [67] "https://github.com/geecko86/QuickLyric"                            
# [68] "https://github.com/alphazero/jredis" -> Travis                                
# [69] "https://github.com/jd-alexander/Google-Directions-Android" -> Travis          
# [70] "https://github.com/GeoLatte/geolatte-geom" -> Travis                          
# [71] "https://github.com/Cognifide/Slice" -> Travis                                
# [72] "https://github.com/AndroidDeveloperLB/AndroidJniBitmapOperations"  
# [73] "https://github.com/lucasr/twoway-view" -> Travis                             
# [74] "https://github.com/harvard-lts/fits"                               
# [75] "https://github.com/lucmoreau/ProvToolbox" -> Travis                          
# [76] "https://github.com/trol73/mucommander" -> Travis                             
# [77] "https://github.com/roundrop/facebook4j" -> Travis                            
# [78] "https://github.com/Tombenpotter/Sanguimancy"                       
# [79] "https://github.com/AEinsam/HabReader"                              
# [80] "https://github.com/yanchenko/droidparts" -> Travis                           
# [81] "https://github.com/dooApp/FXForm2" -> Travis                                
# [82] "https://github.com/WSDOT/wsdot-android-app"                        
# [83] "https://github.com/appium/android-apidemos"                        
# [84] "https://github.com/cinchapi/concourse"                             
# [85] "https://github.com/ganglia/gmetric4j" -> Travis                             
# [86] "https://github.com/ldp4j/ldp4j" -> Travis                                    
# [87] "https://github.com/mikepenz/Android-ActionItemBadge"               
# [88] "https://github.com/RuedigerMoeller/kontraktor" -> Travis                     
# [89] "https://github.com/unic/neba" -> Travis                                      
# [90] "https://github.com/rylinaux/PlugMan" -> Travis                               
# [91] "https://github.com/zeromq/jzmq-api" -> Travis                                
# [92] "https://github.com/spullara/redis-protocol"                        
# [93] "https://github.com/metamx/java-util" -> Travis                               
# [94] "https://github.com/sarl/sarl" -> Travis                                      
# [95] "https://github.com/janus-project/janusproject" -> Project moved                    
# [96] "https://github.com/mayocat/mayocat-shop"                           
# [97] "https://github.com/javasoze/clue"                                  
# [98] "https://github.com/Nerdwin15/stash-jenkins-postreceive-webhook"    
# [99] "https://github.com/calvinaquino/LNReader-Android"                  
#[100] "https://github.com/Orange-OpenSource/ATK" -> Travis                         
#[101] "https://github.com/the-blue-alliance/the-blue-alliance-android" -> Travis    
#[102] "https://github.com/blazsolar/HorizontalPicker"                     
#[103] "https://github.com/Netflix/genie" -> Travis                                  
#[104] "https://github.com/jenkinsci/sbt-plugin"                           
#[105] "https://github.com/michiard/CLOUDS-LAB"                            
#[106] "https://github.com/benas/jPopulator" -> Travis                               
#[107] "https://github.com/vasl-developers/vasl"                           
#[108] "https://github.com/l0rdn1kk0n/wicket-webjars" -> Travis                     
#[109] "https://github.com/holgerbrandl/themoviedbapi" -> Travis                     
#[110] "https://github.com/t-oster/LibLaserCut" -> Travis                            
#[111] "https://github.com/bedatadriven/activityinfo"                      
#[112] "https://github.com/gcacace/android-signaturepad"                   
#[113] "https://github.com/bitbar/testdroid-samples"                       
#[114] "https://github.com/Genymobile/genymotion-binocle"                  
#[115] "https://github.com/melix/deck2pdf" -> Travis                                 
#[116] "https://github.com/jcarolus/android-chess"                         
#[117] "https://github.com/everit-org/jira-timetracker-plugin" -> Travis            
#[118] "https://github.com/jenkinsci/postbuildscript-plugin"                
#[119] "https://github.com/lessthanoptimal/GeoRegression" -> Travis                  
#[120] "https://github.com/libgdx/libgdx-demo-superjumper"                 
#[121] "https://github.com/asterics/AsTeRICS" -> Travis                              
#[122] "https://github.com/ow2-chameleon/fuchsia"                          
#[123] "https://github.com/hanspeide/anytextview"                          
#[124] "https://github.com/sarahtattersall/PIPE" -> Travis                           
#[125] "https://github.com/Gwindow/WhatAndroid"                            
#[126] "https://github.com/andrew-hoffman/halfnes" -> Travis                         
#[127] "https://github.com/afranken/jmeter-analysis-maven-plugin" -> Travis          
#[128] "https://github.com/marvinlabs/android-intents"                     
#[129] "https://github.com/traff/pty4j"                                    
#[130] "https://github.com/Redpill-Linpro/alfresco-libreoffice-online-edit"

# select projects without CI config files
projects_no_ci <- c(
  "socialsensor_graphdb-benchmarks",
  "SharedKnowledge_SharkFW",
  "bigapple520520_bigapple",
  "ai212983_android-spinnerwheel",
  "MassiveCraft_Factions",
  "mihnita_ansi-econsole",
  "janinko_ghprb",
  "Quetzi_Morpheus",
  "MovingBlocks_TerasologyLauncher",
  "pcingola_SnpEff",
  "chrisjenx_Calligraphy",
  "iPaulPro_aFileChooser",
  "wikimedia_apps-android-wikipedia",
  "Todd-Davies_ProgressWheel",
  "couchbase_couchbase-lite-android",
  "gluster_glusterfs-hadoop",
  "perlundq_yajsync",
  "romannurik_muzei",
  "mcharmas_Android-ReactiveLocation",
  "onepf_openiab",
  "cloudfoundry_java-test-applications",
  "hdodenhof_CircleImageView",
  "jbosstm_narayana",
  "versioneye_versioneye_maven_plugin", # underscore in versioneye_maven_plugin should not be replaced
  "arx-deidentifier_arx",
  "OpenLMIS_open-lmis",
  "greenlaw110_Rythm",
  "openshift_jenkins-cloud-plugin",
  "greese_dasein-cloud-core",
  "geecko86_QuickLyric",
  "AndroidDeveloperLB_AndroidJniBitmapOperations",
  "harvard-lts_fits",
  "Tombenpotter_Sanguimancy",
  "AEinsam_HabReader",
  "WSDOT_wsdot-android-app",
  "appium_android-apidemos",
  "cinchapi_concourse",
  "mikepenz_Android-ActionItemBadge",
  "spullara_redis-protocol",
  "mayocat_mayocat-shop",
  "javasoze_clue",
  "Nerdwin15_stash-jenkins-postreceive-webhook",
  "calvinaquino_LNReader-Android",
  "blazsolar_HorizontalPicker",
  "jenkinsci_sbt-plugin",
  "michiard_CLOUDS-LAB",
  "vasl-developers_vasl",
  "bedatadriven_activityinfo",
  "gcacace_android-signaturepad",
  "bitbar_testdroid-samples",
  "Genymobile_genymotion-binocle",
  "jcarolus_android-chess",
  "jenkinsci_postbuildscript-plugin",
  "libgdx_libgdx-demo-superjumper",
  "ow2-chameleon_fuchsia",
  "hanspeide_anytextview",
  "Gwindow_WhatAndroid",
  "marvinlabs_android-intents",
  "traff_pty4j",
  "Redpill-Linpro_alfresco-libreoffice-online-edit"
)
length(projects_no_ci)
# 60

length(intersect(projects_before_after, projects_no_ci))
# 60

projects_before_after <- projects_no_ci

# programming language of selected projects

table(projects_comparison[projects_comparison$repo_name %in% gsub("_", "/", projects_before_after) | projects_comparison$repo_name == "versioneye/versioneye_maven_plugin",]$language)
# Java 
#   60 

commits_default_branch_before_mid <- commits_default_branch_before_mid[commits_default_branch_before_mid$project %in% projects_before_after,]
commits_default_branch_after_mid <- commits_default_branch_after_mid[commits_default_branch_after_mid$project %in% projects_before_after,]

merges_default_branch_before_mid <- merges_default_branch_before_mid[merges_default_branch_before_mid$project %in% projects_before_after,]
merges_default_branch_after_mid <- merges_default_branch_after_mid[merges_default_branch_after_mid$project %in% projects_before_after,]

# analyze merge ratio

merge_ratio_before_mid <- get_project_merge_ratio(commits_default_branch_before_mid, merges_default_branch_before_mid)
merge_ratio_after_mid <- get_project_merge_ratio(commits_default_branch_after_mid, merges_default_branch_after_mid)

merge_ratio_before_after_mid <- merge(merge_ratio_before_mid, merge_ratio_after_mid, by="project")
names(merge_ratio_before_after_mid) <- c("project", "before", "after")

summary(merge_ratio_before_after_mid$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.00106 0.04079 0.11882 0.15674 0.21528 0.50000
IQR(merge_ratio_before_after_mid$before)
# 0.1744845

summary(merge_ratio_before_after_mid$after)
# Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# 0.001368 0.060033 0.188965 0.220913 0.343510 0.500000 
IQR(merge_ratio_before_after_mid$after)
# 0.2834772

wilcox.test(merge_ratio_before_after_mid$after,
            merge_ratio_before_after_mid$before,
            alternative="two.sided", paired=T, correct=T, exact=F) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# V = 1168, p-value = 0.006742
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(merge_ratio_before_after_mid$after,
        merge_ratio_before_after_mid$before,
        paired=TRUE
)
# delta estimate: 0.2025 (small)
# 95 percent confidence interval:
#   inf          sup 
# -0.008124102  0.395919393 

cohen.d(merge_ratio_before_after_mid$after,
        merge_ratio_before_after_mid$before,
        paired=TRUE
)
# d estimate: 0.4075798 (small)
# 95 percent confidence interval:
#   inf        sup 
# 0.04229874 0.77286093


#library(vioplot)
devtools::install_github("TomKellyGenetics/vioplotx")
pdf("figures/comparison_sample_merge_ratio_violin.pdf", bg="transparent", width=8, height=6)
par(
  bg="white",
  #mar = c(3, 3, 3, 1)+0.1, # subplot margins (bottom, left, top, right)
  #  omi = c(0.0, 0.0, 0.0, 0.0),  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.1,
  cex.main=1.3,
  cex.sub=1.1,
  cex.lab=1,1,
  cex.axis=1.1
)

vioplotx::vioplotx(
  merge_ratio_before_after_mid$before,
  merge_ratio_before_after_mid$after,
  col="gray90",
  pchMed=4,
  rectCol="white",
  colMed="black",
  horizontal=FALSE,
  cex.names=1.3,
  wex=1,
  names=c("before_mid", "after_mid"),
  ylim=c(0, 0.7),
  yaxt="n"
)
#abline(v=median(project_pr_ratio_default_branch_before_during$during), lty=3)
segments(1, median(merge_ratio_before_after_mid$before),
         0, median(merge_ratio_before_after_mid$before),
         lty=3
)
segments(2, median(merge_ratio_before_after_mid$after),
         0, median(merge_ratio_before_after_mid$after),
         lty=3
)
axis(2, at=seq(0, 0.7, by=0.1), labels=seq(0, 0.7, by=0.1), las=2)
title("Merge ratio (comparison sample)")

par(mfrow=c(1, 1))
dev.off()

############
# Analyze pull-request ratio 
############
project_pr_ratio_default_branch_before_mid <- get_project_pr_ratio(merges_default_branch_before_mid)
project_pr_ratio_default_branch_after_mid <- get_project_pr_ratio(merges_default_branch_after_mid)

project_pr_ratio_default_branch_before_after <- merge(project_pr_ratio_default_branch_before_mid, project_pr_ratio_default_branch_after_mid, by="project")
names(project_pr_ratio_default_branch_before_after) <- c("project", "before", "after")

summary(project_pr_ratio_default_branch_before_after$before)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0000  0.0750  0.8579  0.6184  1.0000  1.0000
IQR(project_pr_ratio_default_branch_before_after$before)
# 0.925

summary(project_pr_ratio_default_branch_before_after$after)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0000  0.2875  0.8056  0.6470  1.0000  1.0000 
IQR(project_pr_ratio_default_branch_before_after$after)
# 0.7125

wilcox.test(project_pr_ratio_default_branch_before_after$after,
            project_pr_ratio_default_branch_before_after$before,
            alternative="two.sided", paired=T, correct=T, exact=F) # Wilcoxon rank sum test (equivalent to the Mann-Whitney test), exact distribution (Streitberg-Roehmel algorithm) => handles ties
# V = 285, p-value = 0.7013
# alternative hypothesis: true location shift is not equal to 0

cliff.delta(project_pr_ratio_default_branch_before_after$after,
        project_pr_ratio_default_branch_before_after$before,
        paired=TRUE
)
# delta estimate: -0.01472222 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.2121730  0.1838834

cohen.d(project_pr_ratio_default_branch_before_after$after,
        project_pr_ratio_default_branch_before_after$before,
        paired=TRUE
)
# d estimate: 0.07766983 (negligible)
# 95 percent confidence interval:
#   inf        sup 
# -0.2840131  0.439352

###################################################


##########
# Analyze default branch build ratio
##########
# read data for commits to default branch in sample projects during CI
branch_builds <- fread("data/tr_projects_sample_filtered_2_branch_builds.csv", header=TRUE, sep=",", quote="\"", strip.white=TRUE, showProgress=TRUE, encoding="UTF-8", na.strings=c("", "null"))

n <- sum(branch_builds$build_count_default_branch+branch_builds$build_count_other_branches)
n_default <- sum(branch_builds$build_count_default_branch)
n_default/n*100
# 81.84901

ratio <- branch_builds$build_count_default_branch / (branch_builds$build_count_default_branch+branch_builds$build_count_other_branches)
summary(ratio)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0874  0.7158  0.8595  0.8068  0.9076  0.9985
IQR(ratio)
# 0.191791
boxplot(ratio)
